==> phoenix6-24.0.0b3-cp39-abi3-win_amd64.whl: phoenix6-24.0.0b3.data/purelib/phoenix6/controls/differential_voltage.py <==
"""
Copyright (C) Cross The Road Electronics.  All rights reserved.
License information can be found in CTRE_LICENSE.txt
For support and suggestions contact support@ctr-electronics.com or file
an issue tracker at https://github.com/CrossTheRoadElec/Phoenix-Releases
"""

from phoenix6.phoenix_native import Native
from phoenix6.status_code import StatusCode
from phoenix6.units import *
import ctypes


class DifferentialVoltage:
    """
    Request a specified voltage with a differential position closed-loop.
    
    This control mode will attempt to apply the specified voltage to the motor. If
    the supply voltage is below the requested voltage, the motor controller will
    output the supply voltage. It will also set the motor's differential position
    setpoint to the specified position.
    
    :param target_output:    Voltage to attempt to drive at
    :type target_output: volt
    :param differential_position:    Differential position to drive towards in
                                     rotations
    :type differential_position: rotation
    :param enable_foc:    Set to true to use FOC commutation (requires Phoenix Pro),
                          which increases peak power by ~15%. Set to false to use
                          trapezoidal commutation.  FOC improves motor performance
                          by leveraging torque (current) control.  However, this may
                          be inconvenient for applications that require specifying
                          duty cycle or voltage.  CTR-Electronics has developed a
                          hybrid method that combines the performances gains of FOC
                          while still allowing applications to provide duty cycle or
                          voltage demand.  This not to be confused with simple
                          sinusoidal control or phase voltage control which lacks
                          the performance gains.
    :type enable_foc: bool
    :param differential_slot:    Select which gains are applied to the differential
                                 controller by selecting the slot.  Use the
                                 configuration api to set the gain values for the
                                 selected slot before enabling this feature. Slot
                                 must be within [0,2].
    :type differential_slot: int
    :param override_brake_dur_neutral:    Set to true to static-brake the rotor when
                                          output is zero (or within deadband).  Set
                                          to false to use the NeutralMode
                                          configuration setting (default). This flag
                                          exists to provide the fundamental behavior
                                          of this control when output is zero, which
                                          is to provide 0V to the motor.
    :type override_brake_dur_neutral: bool
    """

    def __init__(self, target_output: volt, differential_position: rotation, enable_foc: bool = True, differential_slot: int = 1, override_brake_dur_neutral: bool = False):
        self._name = "DifferentialVoltage"
        self.update_freq_hz: hertz = 100.0
        
        self.target_output = target_output
        """
        Voltage to attempt to drive at
        """
        self.differential_position = differential_position
        """
        Differential position to drive towards in rotations
        """
        self.enable_foc = enable_foc
        """
        Set to true to use FOC commutation (requires Phoenix Pro), which increases peak
        power by ~15%. Set to false to use trapezoidal commutation.  FOC improves motor
        performance by leveraging torque (current) control.  However, this may be
        inconvenient for applications that require specifying duty cycle or voltage. 
        CTR-Electronics has developed a hybrid method that combines the performances
        gains of FOC while still allowing applications to provide duty cycle or voltage
        demand.  This not to be confused with simple sinusoidal control or phase voltage
        control which lacks the performance gains.
        """
        self.differential_slot = differential_slot
        """
        Select which gains are applied to the differential controller by selecting the
        slot.  Use the configuration api to set the gain values for the selected slot
        before enabling this feature. Slot must be within [0,2].
        """
        self.override_brake_dur_neutral = override_brake_dur_neutral
        """
        Set to true to static-brake the rotor when output is zero (or within deadband). 
        Set to false to use the NeutralMode configuration setting (default). This flag
        exists to provide the fundamental behavior of this control when output is zero,
        which is to provide 0V to the motor.
        """

    @property
    def name(self) -> str:
        """
        Gets the name of this control request.

        :returns: Name of the control request
        :rtype: str
        """
        return self._name

    def __str__(self) -> str:
        ss = []
        ss.append("class: DifferentialVoltage")
        ss.append("target_output: " + str(self.target_output))
        ss.append("differential_position: " + str(self.differential_position))
        ss.append("enable_foc: " + str(self.enable_foc))
        ss.append("differential_slot: " + str(self.differential_slot))
        ss.append("override_brake_dur_neutral: " + str(self.override_brake_dur_neutral))
        return "\n".join(ss)

    def _send_request(self, network: str, device_hash: int, cancel_other_requests: bool) -> StatusCode:
        """
        Sends this request out over CAN bus to the device for
        the device to apply.

        :param network: Network to send request over
        :type network: str
        :param device_hash: Device to send request to
        :type device_hash: int
        :param cancel_other_requests: True to cancel other requests
        :type cancel_other_requests: bool
        :returns: Status of the send operation
        :rtype: StatusCode
        """

        return StatusCode(Native.instance().c_ctre_phoenix6_RequestControlDifferentialVoltage(ctypes.c_char_p(bytes(network, 'utf-8')), device_hash, self.update_freq_hz, cancel_other_requests, self.target_output, self.differential_position, self.enable_foc, self.differential_slot, self.override_brake_dur_neutral))

    
    def with_target_output(self, new_target_output: volt) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's target_output parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_target_output: Parameter to modify
        :type new_target_output: volt
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.target_output = new_target_output
        return self
    
    def with_differential_position(self, new_differential_position: rotation) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's differential_position parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_differential_position: Parameter to modify
        :type new_differential_position: rotation
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.differential_position = new_differential_position
        return self
    
    def with_enable_foc(self, new_enable_foc: bool) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's enable_foc parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_enable_foc: Parameter to modify
        :type new_enable_foc: bool
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.enable_foc = new_enable_foc
        return self
    
    def with_differential_slot(self, new_differential_slot: int) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's differential_slot parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_differential_slot: Parameter to modify
        :type new_differential_slot: int
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.differential_slot = new_differential_slot
        return self
    
    def with_override_brake_dur_neutral(self, new_override_brake_dur_neutral: bool) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's override_brake_dur_neutral parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_override_brake_dur_neutral: Parameter to modify
        :type new_override_brake_dur_neutral: bool
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.override_brake_dur_neutral = new_override_brake_dur_neutral
        return self

    def with_update_freq_hz(self, new_update_freq_hz: hertz) -> 'DifferentialVoltage':
        """
        Sets the period at which this control will update at.
        This is designated in Hertz, with a minimum of 20 Hz
        (every 50 ms) and a maximum of 1000 Hz (every 1 ms).

        If this field is set to 0 Hz, the control request will
        be sent immediately as a one-shot frame. This may be useful
        for advanced applications that require outputs to be
        synchronized with data acquisition. In this case, we
        recommend not exceeding 50 ms between control calls.

        :param new_update_freq_hz: Parameter to modify
        :type new_update_freq_hz: hertz
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.update_freq_hz = new_update_freq_hz
        return self

    @property
    def control_info(self) -> dict:
        """
        Gets information about this control request.

        :returns: Dictonary of control parameter names and corresponding applied values
        :rtype: dict
        """
        control_info = {}
        control_info["name"] = self._name
        control_info["target_output"] = self.target_output
        control_info["differential_position"] = self.differential_position
        control_info["enable_foc"] = self.enable_foc
        control_info["differential_slot"] = self.differential_slot
        control_info["override_brake_dur_neutral"] = self.override_brake_dur_neutral
        return control_info

==> phoenix6-24.2.0-cp39-abi3-win_amd64.whl: phoenix6-24.2.0.data/purelib/phoenix6/controls/differential_voltage.py <==
"""
Copyright (C) Cross The Road Electronics.  All rights reserved.
License information can be found in CTRE_LICENSE.txt
For support and suggestions contact support@ctr-electronics.com or file
an issue tracker at https://github.com/CrossTheRoadElec/Phoenix-Releases
"""

from phoenix6.phoenix_native import Native
from phoenix6.status_code import StatusCode
from phoenix6.units import *
import ctypes


class DifferentialVoltage:
    """
    Request a specified voltage with a differential position closed-loop.
    
    This control mode will attempt to apply the specified voltage to the motor. If
    the supply voltage is below the requested voltage, the motor controller will
    output the supply voltage. It will also set the motor's differential position
    setpoint to the specified position.
    
    :param target_output:    Voltage to attempt to drive at
    :type target_output: volt
    :param differential_position:    Differential position to drive towards in
                                     rotations
    :type differential_position: rotation
    :param enable_foc:    Set to true to use FOC commutation (requires Phoenix Pro),
                          which increases peak power by ~15%. Set to false to use
                          trapezoidal commutation.  FOC improves motor performance
                          by leveraging torque (current) control.  However, this may
                          be inconvenient for applications that require specifying
                          duty cycle or voltage.  CTR-Electronics has developed a
                          hybrid method that combines the performances gains of FOC
                          while still allowing applications to provide duty cycle or
                          voltage demand.  This not to be confused with simple
                          sinusoidal control or phase voltage control which lacks
                          the performance gains.
    :type enable_foc: bool
    :param differential_slot:    Select which gains are applied to the differential
                                 controller by selecting the slot.  Use the
                                 configuration api to set the gain values for the
                                 selected slot before enabling this feature. Slot
                                 must be within [0,2].
    :type differential_slot: int
    :param override_brake_dur_neutral:    Set to true to static-brake the rotor when
                                          output is zero (or within deadband).  Set
                                          to false to use the NeutralMode
                                          configuration setting (default). This flag
                                          exists to provide the fundamental behavior
                                          of this control when output is zero, which
                                          is to provide 0V to the motor.
    :type override_brake_dur_neutral: bool
    :param limit_forward_motion:    Set to true to force forward limiting.  This
                                    allows users to use other limit switch sensors
                                    connected to robot controller.  This also allows
                                    use of active sensors that require external
                                    power.
    :type limit_forward_motion: bool
    :param limit_reverse_motion:    Set to true to force reverse limiting.  This
                                    allows users to use other limit switch sensors
                                    connected to robot controller.  This also allows
                                    use of active sensors that require external
                                    power.
    :type limit_reverse_motion: bool
    """

    def __init__(self, target_output: volt, differential_position: rotation, enable_foc: bool = True, differential_slot: int = 1, override_brake_dur_neutral: bool = False, limit_forward_motion: bool = False, limit_reverse_motion: bool = False):
        self._name = "DifferentialVoltage"
        self.update_freq_hz: hertz = 100.0
        
        self.target_output = target_output
        """
        Voltage to attempt to drive at
        """
        self.differential_position = differential_position
        """
        Differential position to drive towards in rotations
        """
        self.enable_foc = enable_foc
        """
        Set to true to use FOC commutation (requires Phoenix Pro), which increases peak
        power by ~15%. Set to false to use trapezoidal commutation.  FOC improves motor
        performance by leveraging torque (current) control.  However, this may be
        inconvenient for applications that require specifying duty cycle or voltage. 
        CTR-Electronics has developed a hybrid method that combines the performances
        gains of FOC while still allowing applications to provide duty cycle or voltage
        demand.  This not to be confused with simple sinusoidal control or phase voltage
        control which lacks the performance gains.
        """
        self.differential_slot = differential_slot
        """
        Select which gains are applied to the differential controller by selecting the
        slot.  Use the configuration api to set the gain values for the selected slot
        before enabling this feature. Slot must be within [0,2].
        """
        self.override_brake_dur_neutral = override_brake_dur_neutral
        """
        Set to true to static-brake the rotor when output is zero (or within deadband). 
        Set to false to use the NeutralMode configuration setting (default). This flag
        exists to provide the fundamental behavior of this control when output is zero,
        which is to provide 0V to the motor.
        """
        self.limit_forward_motion = limit_forward_motion
        """
        Set to true to force forward limiting.  This allows users to use other limit
        switch sensors connected to robot controller.  This also allows use of active
        sensors that require external power.
        """
        self.limit_reverse_motion = limit_reverse_motion
        """
        Set to true to force reverse limiting.  This allows users to use other limit
        switch sensors connected to robot controller.  This also allows use of active
        sensors that require external power.
        """

    @property
    def name(self) -> str:
        """
        Gets the name of this control request.

        :returns: Name of the control request
        :rtype: str
        """
        return self._name

    def __str__(self) -> str:
        ss = []
        ss.append("class: DifferentialVoltage")
        ss.append("target_output: " + str(self.target_output))
        ss.append("differential_position: " + str(self.differential_position))
        ss.append("enable_foc: " + str(self.enable_foc))
        ss.append("differential_slot: " + str(self.differential_slot))
        ss.append("override_brake_dur_neutral: " + str(self.override_brake_dur_neutral))
        ss.append("limit_forward_motion: " + str(self.limit_forward_motion))
        ss.append("limit_reverse_motion: " + str(self.limit_reverse_motion))
        return "\n".join(ss)

    def _send_request(self, network: str, device_hash: int, cancel_other_requests: bool) -> StatusCode:
        """
        Sends this request out over CAN bus to the device for
        the device to apply.

        :param network: Network to send request over
        :type network: str
        :param device_hash: Device to send request to
        :type device_hash: int
        :param cancel_other_requests: True to cancel other requests
        :type cancel_other_requests: bool
        :returns: Status of the send operation
        :rtype: StatusCode
        """
        return StatusCode(Native.instance().c_ctre_phoenix6_RequestControlDifferentialVoltage(ctypes.c_char_p(bytes(network, 'utf-8')), device_hash, self.update_freq_hz, cancel_other_requests, self.target_output, self.differential_position, self.enable_foc, self.differential_slot, self.override_brake_dur_neutral, self.limit_forward_motion, self.limit_reverse_motion))

    
    def with_target_output(self, new_target_output: volt) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's target_output parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_target_output: Parameter to modify
        :type new_target_output: volt
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.target_output = new_target_output
        return self
    
    def with_differential_position(self, new_differential_position: rotation) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's differential_position parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_differential_position: Parameter to modify
        :type new_differential_position: rotation
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.differential_position = new_differential_position
        return self
    
    def with_enable_foc(self, new_enable_foc: bool) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's enable_foc parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_enable_foc: Parameter to modify
        :type new_enable_foc: bool
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.enable_foc = new_enable_foc
        return self
    
    def with_differential_slot(self, new_differential_slot: int) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's differential_slot parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_differential_slot: Parameter to modify
        :type new_differential_slot: int
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.differential_slot = new_differential_slot
        return self
    
    def with_override_brake_dur_neutral(self, new_override_brake_dur_neutral: bool) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's override_brake_dur_neutral parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_override_brake_dur_neutral: Parameter to modify
        :type new_override_brake_dur_neutral: bool
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.override_brake_dur_neutral = new_override_brake_dur_neutral
        return self
    
    def with_limit_forward_motion(self, new_limit_forward_motion: bool) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's limit_forward_motion parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_limit_forward_motion: Parameter to modify
        :type new_limit_forward_motion: bool
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.limit_forward_motion = new_limit_forward_motion
        return self
    
    def with_limit_reverse_motion(self, new_limit_reverse_motion: bool) -> 'DifferentialVoltage':
        """
        Modifies this Control Request's limit_reverse_motion parameter and returns itself for
        method-chaining and easier to use request API.
    
        :param new_limit_reverse_motion: Parameter to modify
        :type new_limit_reverse_motion: bool
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.limit_reverse_motion = new_limit_reverse_motion
        return self

    def with_update_freq_hz(self, new_update_freq_hz: hertz) -> 'DifferentialVoltage':
        """
        Sets the period at which this control will update at.
        This is designated in Hertz, with a minimum of 20 Hz
        (every 50 ms) and a maximum of 1000 Hz (every 1 ms).

        If this field is set to 0 Hz, the control request will
        be sent immediately as a one-shot frame. This may be useful
        for advanced applications that require outputs to be
        synchronized with data acquisition. In this case, we
        recommend not exceeding 50 ms between control calls.

        :param new_update_freq_hz: Parameter to modify
        :type new_update_freq_hz: hertz
        :returns: Itself
        :rtype: DifferentialVoltage
        """
        self.update_freq_hz = new_update_freq_hz
        return self

    @property
    def control_info(self) -> dict:
        """
        Gets information about this control request.

        :returns: Dictonary of control parameter names and corresponding applied values
        :rtype: dict
        """
        control_info = {}
        control_info["name"] = self._name
        control_info["target_output"] = self.target_output
        control_info["differential_position"] = self.differential_position
        control_info["enable_foc"] = self.enable_foc
        control_info["differential_slot"] = self.differential_slot
        control_info["override_brake_dur_neutral"] = self.override_brake_dur_neutral
        control_info["limit_forward_motion"] = self.limit_forward_motion
        control_info["limit_reverse_motion"] = self.limit_reverse_motion
        return control_info
